import 'dart:convert';

import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/account_details_bean.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/contact_details_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Models/notes_details_bean.dart';
import 'package:example/Models/task_detail_bean.dart';
import 'package:example/Screens/CRM/CreateNoteScreen.dart';
import 'package:example/Screens/CRM/CreateTaskScreen.dart';
import 'package:example/Screens/CRM/platform.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/JsonFormComponent/json_schema.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:transition/transition.dart';

class ContactViewDetailScreen extends StatefulWidget {
  String pageTitle;
  String email;
  String firstName;
  String companyName;
  String id;
  String type;
  Color theme;
  Color secondaryColor;
  Function() onCallBack;
  String dateTime;
  ContactListBean contactListDetails;

  ContactViewDetailScreen({
    Key key,
    this.pageTitle,
    this.theme,
    this.type,
    this.id,
    this.secondaryColor,
    this.contactListDetails,
    this.onCallBack,
    this.companyName,
    this.dateTime,
  }) : super(key: key);

  @override
  _ContactViewDetailScreenState createState() =>
      _ContactViewDetailScreenState();
}

class _ContactViewDetailScreenState extends State<ContactViewDetailScreen> {
  TaskDetailBean taskDetailBean;

  List<TaskDetailList> taskListBeanData = new List.empty(growable: true);
  NotesDetailsBean notesDetailsBean;
  List<NotesListDetailsBean> notesListDetailsBean =
      new List.empty(growable: true);
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  dynamic formGeneral;
  List<dynamic> status = new List.empty(growable: true);
  List<dynamic> account = new List.empty(growable: true);
  List<dynamic> sales = new List.empty(growable: true);
  String form;
  dynamic response;
  bool isLoader = true;
  bool isLoading = true;
  Map decorations;
  Map validations = {'text1': validationExample};
  Map errorText = {
    'input1': "Error Value",
    'text1': "Enter the Password",
  };

  static String validationExample(field, value) {
    String pattern =
        r'^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[!@#\$&*~]).{8,}$';
    RegExp regExp = new RegExp(pattern);

    if (value.isEmpty) {
      return 'Please enter some text';
    }
    if (value.length < 8) {
      return 'Password is too short';
    }

    if (!regExp.hasMatch(value)) {
      return 'Password contains Minimum 1 Upper case Minimum 1 lowercase Minimum 1 Numeric Number Minimum 1 Special Character Common Allow Character ( ! @ # \$ & * ~ )';
    }
    return null;
  }

  List<String> taskName = ["Open", "Close"];
  bool flag = true;

  bool showFloating = false;
  int indexValue;

  AccountDetailsBean accountDetailsBean;

  static List<AccountListBean> accountListBeanData =
      new List.empty(growable: true);

  @override
  void initState() {
    super.initState();

    contactMastViewAPI();

    initSharedPref();
    if (widget.type != "Add") {
      taskAPI();
      notesAPI();
    }
  }

  String ownerId;

  Future<void> initSharedPref() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        key: _scaffoldKey,
        appBar: widget.type == "Add"
            ? AppBar(
                title: Text("Add Contact"),
                backgroundColor: widget.theme,
              )
            : null,
        body: widget.type != "Add"
            ? DefaultTabController(
                length: 3,
                child: NestedScrollView(
                    headerSliverBuilder:
                        (BuildContext context, bool innerBoxIsScrolled) {
                      return <Widget>[
                        SliverAppBar(
                          expandedHeight: 150.0,
                          floating: true,
                          pinned: true,
                          title: Text(
                              widget.contactListDetails.accountid.companyname ??
                                  widget.pageTitle),
                          backgroundColor:
                              widget.theme ?? AppColors.APP_COLOR_PRIMARY,
                          bottom: TabBar(
                            onTap: (index) {
                              indexValue = index;
                              if (index == 1) {
                                setState(() {
                                  showFloating = true;
                                });
                              } else if (index == 2) {
                                setState(() {
                                  showFloating = true;
                                });
                              } else {
                                setState(() {
                                  showFloating = false;
                                });
                              }
                            },
                            indicatorColor:
                                widget.theme ?? AppColors.APP_COLOR_PRIMARY,
                            labelColor: Colors.black87,
                            unselectedLabelColor: Colors.grey,
                            tabs: [
                              Tab(
                                child: Container(
                                  child: Text(
                                    AppStrings.CONTACT_INFO,
                                    style: TextStyle(
                                        color: AppColors.APP_COLOR_WHITE,
                                        fontSize: FontSize.FONT_15,
                                        fontWeight: FontWeight.bold),
                                  ),
                                ),
                              ),
                              Tab(
                                child: Container(
                                  child: Text(
                                    AppStrings.TASK,
                                    style: TextStyle(
                                        color: AppColors.APP_COLOR_WHITE,
                                        fontSize: FontSize.FONT_15,
                                        fontWeight: FontWeight.bold),
                                  ),
                                ),
                              ),
                              Tab(
                                child: Container(
                                  child: Text(
                                    AppStrings.NOTES,
                                    style: TextStyle(
                                        color: AppColors.APP_COLOR_WHITE,
                                        fontSize: FontSize.FONT_15,
                                        fontWeight: FontWeight.bold),
                                  ),
                                ),
                              ),
                            ],
                          ),
                        ),
                      ];
                    },
                    body: isLoading
                        ? Center(
                            child: Loader(
                            loaderColor: widget.theme,
                          ))
                        : TabBarView(
                            physics: NeverScrollableScrollPhysics(),
                            children: [
                              _formView,
                              _taskView,
                              _notesView,
                            ],
                          )),
              )
            : isLoading
                ? Center(
                    child: Loader(
                    loaderColor: widget.theme,
                  ))
                : account.length != 0
                    ? Container(child: _formView)
                    : Center(
                        child: Loader(
                        loaderColor: widget.theme,
                      )),
        floatingActionButton: showFloating
            ? FloatingActionButton(
                onPressed: () {
                  if (indexValue == 1) {
                    Navigator.push(
                        context,
                        Transition(
                            child: CreateTaskScreen(
                              id: widget.id.toString(),
                              primaryColor: widget.theme,
                              secondaryColor: widget.secondaryColor,
                              onCallBack: onTaskAPICall,
                              fromPageId: 'contactid',
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                  } else {
                    Navigator.push(
                        context,
                        Transition(
                            child: CreateNoteScreen(
                              id: widget.id.toString(),
                              primaryColor: widget.theme,
                              secondaryColor: widget.secondaryColor,
                              onCallBack: onNotesAPICall,
                              fromPageId: 'contactid',
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                  }
                },
                child: const Icon(Icons.add),
                backgroundColor: widget.theme ?? Colors.green,
              )
            : null,
      ),
    );
  }

  Widget get _formView {
    return new SingleChildScrollView(
        child: Form(
      child: new Center(
        child: new Column(children: <Widget>[
          new JsonSchema(
            primaryColor: widget.theme ?? AppColors.primaryColor,
            secondaryColor: widget.secondaryColor ?? AppColors.primaryColor,
            form: form,
            validations: validations,
            errorMessages: errorText,
            buttonText: widget.type == AppStrings.ADD
                ? AppStrings.ADD
                : AppStrings.SAVE,
            onChanged: (dynamic response) {
              this.response = response;
            },
            actionSave: (data) {
              Utils.hideKeyboard(context);
              setState(() {
                formGeneral = data;
              });

              var jsonObj;
              if (widget.type != "Add") {
                jsonObj = {
                  for (int i = 0; i < data['fields'].length; i++)
                    json.encode(data['fields'][i]['apiKeyValue'].toString()):
                        json.encode(data['fields'][i]['value'].toString())
                };
              } else {
                jsonObj = {
                  for (int i = 0; i < data['fields'].length; i++)
                    json.encode(data['fields'][i]['apiKeyValue'].toString()):
                        json.encode(data['fields'][i]['value'].toString()),
                  "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
                  "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
                  "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
                };
              }

              if (widget.type != "Add") {
                updateContactAPI(jsonObj.toString());
              } else {
                addContactAPI(jsonObj.toString());
              }
            },
            actionCancel: () {
              Navigator.pop(context);
            },
            buttonSave: true,
          ),
        ]),
      ),
    ));
  }

  Widget get _taskView {
    if (taskListBeanData.length != 0) {
      return ListView.builder(
        itemCount: taskName.length,
        itemBuilder: (_, index) {
          return ExpansionTile(
              title: Text(
                taskName[index].toString() == "Open"
                    ? "UpComing Task"
                    : taskName[index].toString() == "Close"
                        ? "Completed Task"
                        : taskName[index].toString(),
                style: TextStyle(
                    color: AppColors.APP_COLOR_BLACK,
                    fontSize: FontSize.FONT_17,
                    fontWeight: FontWeight.bold),
              ),
              children: [
                ...taskListBeanData.map((e) {
                  if (e.statusid.name == taskName[index]) {
                    return BUICard.taskCard(
                      cardStatus: e.activitytypeid.name,
                      cardDesc: e.desc,
                      cardDate: widget.dateTime == "dd-mm-yyyy" &&
                              widget.dateTime != null
                          ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(e.duedate)}" ??
                              ""
                          : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(e.duedate)}" ??
                              "",
                      cardType: CardType.cardTask,
                      cardSize: CardSize.medium,
                      cardColor: Colors.white,
                      cardTextColor: Colors.black87,
                      width: MediaQuery.of(context).size.width,
                      padding: 5,
                      height: 140,
                      isActiveStatus: e.statusid.name == "Open" ? false : true,
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(15.0),
                      ),
                      elevation: 10,
                      secondaryColor: widget.secondaryColor,
                      onStateChange: () {
                        setState(() {
                          if (e.statusid.name == "Open") {
                            e.statusid.name = "Close";
                            Utils.showInSnackBar(
                                _scaffoldKey, "Move into Close state");
                          } else {
                            Utils.showInSnackBar(_scaffoldKey,
                                "You cannot change the task from the closed position to the open position. Please contact admin");
                          }
                        });
                      },
                    );
                  } else {
                    return Container();
                  }
                }).toList(),
              ]);
        },
      );
    } else {
      return Center(
        child: Container(
          child: Text(
            AppStrings.NO_TASK_AVAILABLE,
            style: TextStyle(
                color: AppColors.textColor,
                fontSize: FontSize.FONT_20,
                fontWeight: FontWeight.bold),
          ),
        ),
      );
    }
  }

  Widget get _notesView {
    if (notesListDetailsBean.length != 0) {
      return ListView.builder(
          padding: const EdgeInsets.all(8),
          itemCount: notesListDetailsBean.length,
          itemBuilder: (BuildContext context, int index) {
            return BUICard.notesCard(
              cardType: CardType.cardNotes,
              cardSize: CardSize.medium,
              cardColor: Colors.white,
              cardTextColor: Colors.black87,
              width: MediaQuery.of(context).size.width,
              padding: 2,
              height: 120,
              cardAuthor: widget.contactListDetails.firstname,
              cardDesc: notesListDetailsBean[index].desc,
              cardDate: widget.dateTime == "dd-mm-yyyy" &&
                      widget.dateTime != null
                  ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(notesListDetailsBean[index].updatedAt)}" ??
                      ""
                  : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(notesListDetailsBean[index].updatedAt)}" ??
                      "",
              secondaryColor: widget.secondaryColor,
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(5.0),
              ),
              elevation: 10,
              flag: flag,
              onStateChange: () {
                setState(() {
                  flag = !flag;
                });
              },
            );

            /*return BUICard(
              cardType: CardType.customCard,
              cardText: notesListDetailsBean[index].desc.toString(),
              cardSize: CardSize.small,
              height: MediaQuery.of(context).size.height * 0.2,
              cardColor: Colors.white,
              elevation: 20,
              padding: 15,
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(15.0),
              ),
            );*/
          });
    } else {
      return Center(
        child: Container(
          child: Text(
            AppStrings.NO_NOTES_AVAILABLE,
            style: TextStyle(
                color: AppColors.textColor,
                fontSize: FontSize.FONT_20,
                fontWeight: FontWeight.bold),
          ),
        ),
      );
    }
  }

  Future<void> contactMastViewAPI() async {
    setState(() {
      isLoading = true;
    });

    String urlController = ServerUrl.CRM_MASTER_VIEW;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);

    if (detailsBean.statusCode == 400) {
      setState(() {
        isLoader = true;
      });

      MyApp.callLoginAPI(context);
    }
    if (detailsBean.status) {
      status = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Contact")
            if (detailsBean.data[i].key == "Status")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];

      sales = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].key == "Area")
            {
              'label': detailsBean.data[i].name,
              'value': detailsBean.data[i].id.toString(),
            },
      ];

      if (widget.type != "Add") {
        form = json.encode({
          'fields': [
            {
              'apiKeyValue': 'firstname',
              'key': 'First Name',
              'type': 'input',
              'label': 'First Name',
              'required': true,
              'readOnly': false,
              'value': widget.contactListDetails.firstname.toString() != "null"
                  ? widget.contactListDetails.firstname.toString()
                  : "",
            },
            {
              'apiKeyValue': 'secondName',
              'key': 'Second Name',
              'type': 'input',
              'label': 'Second Name',
              'required': true,
              'readOnly': false,
              'value': widget.contactListDetails.secondName.toString() != "null"
                  ? widget.contactListDetails.secondName.toString()
                  : "",
            },
            {
              'apiKeyValue': 'statusid',
              'key': 'Status',
              'type': 'select',
              'label': 'Status',
              'required': true,
              'value': widget.contactListDetails.statusid.id.toString(),
              'items': status != null
                  ? status
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'accountid',
              'key': 'Account',
              'type': 'input',
              'label': 'Account ID',
              'required': true,
              'readOnly': true,
              'value':
                  widget.contactListDetails.accountid.id.toString() != "null"
                      ? widget.contactListDetails.accountid.id.toString()
                      : "",
            },
            {
              'apiKeyValue': 'title',
              'key': 'Title',
              'type': 'input',
              'label': 'Title',
              'required': true,
              'readOnly': false,
              'value': widget.contactListDetails.title.toString() != "null"
                  ? widget.contactListDetails.title.toString()
                  : "",
            },
            {
              'apiKeyValue': 'dept',
              'key': 'Department',
              'type': 'input',
              'label': 'Department',
              'required': true,
              'readOnly': false,
              'value': widget.contactListDetails.dept.toString() != "null"
                  ? widget.contactListDetails.dept.toString()
                  : "",
            },
            {
              'apiKeyValue': 'email',
              'key': 'e-mail',
              'type': 'input',
              'label': 'Email',
              'required': true,
              'readOnly': false,
              'value': widget.contactListDetails.email.toString() != "null"
                  ? widget.contactListDetails.email.toString()
                  : "",
            },
            {
              'apiKeyValue': 'mobile',
              'key': 'mobile',
              'type': 'input',
              'label': 'Mobile no',
              'required': true,
              'readOnly': false,
              'validator': 'number',
              'value': widget.contactListDetails.mobile.toString() != "null"
                  ? widget.contactListDetails.mobile.toString()
                  : "",
            },
            {
              'apiKeyValue': 'phone',
              'key': 'contact',
              'type': 'input',
              'label': 'Phone no',
              'required': true,
              'readOnly': false,
              'validator': 'number',
              'value': widget.contactListDetails.phone.toString() != "null"
                  ? widget.contactListDetails.phone.toString()
                  : "",
            },
            {
              'apiKeyValue': 'area',
              'key': 'Area',
              'type': 'input',
              'label': 'Area',
              'required': true,
              'value': widget.contactListDetails.area.toString() != "null"
                  ? widget.contactListDetails.area.toString()
                  : "",
            },
            {
              'apiKeyValue': 'address1',
              'key': 'Address Line 1',
              'type': 'input',
              'label': 'Address Line 1',
              'required': true,
              'value': widget.contactListDetails.address1.toString() != "null"
                  ? widget.contactListDetails.address1.toString()
                  : "",
            },
            {
              'apiKeyValue': 'address2',
              'key': 'Address Line 2',
              'type': 'input',
              'label': 'Address Line 2',
              'required': false,
              'value': widget.contactListDetails.address2.toString() != "null"
                  ? widget.contactListDetails.address2.toString()
                  : "",
            },
            {
              'apiKeyValue': 'state',
              'key': 'State',
              'type': 'input',
              'label': 'State ',
              'required': true,
              'value': widget.contactListDetails.state.toString() != "null"
                  ? widget.contactListDetails.state.toString()
                  : "",
            },
          ]
        });
      } else {
        setState(() {
          isLoader = true;
          isLoading = true;
        });
        accountScreenViewDetailsAPI();
      }

      setState(() {
        isLoader = false;
        isLoading = false;
      });
    } else {
      setState(() {
        isLoader = false;
        isLoading = false;
      });
    }
  }

  Future<void> updateContactAPI(String jsonString) async {
    String urlController = ServerUrl.CRM_UPDATE_CONTACT_URL;
    var objToSend = {
      "\"condition\"": [
        {"\"id\"": widget.id.toString()}
      ],
      "\"data\"": [jsonString.toString()],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    } else {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
    }
  }

  Future<void> addContactAPI(String jsonString) async {
    String urlController = ServerUrl.CRM_CREATE_CONTACT_URL;

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonString.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      widget.onCallBack();
      Navigator.pop(context);
    } else {}
  }

  Future<void> taskAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_TASK_URL;
    var objToSend = {
      "\"condition\"": [
        {"\"contactId\"": widget.contactListDetails.id.toString()}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TaskDetailBean taskDetailsResponseBean = TaskDetailBean.fromJson(map);

    if (taskDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });
      MyApp.callLoginAPI(context);
    }

    if (taskDetailsResponseBean.status) {
      taskDetailBean = taskDetailsResponseBean;
      taskListBeanData = taskDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> notesAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_NOTES_URL;
    var objToSend = {
      "\"condition\"": [
        {"\"contactId\"": widget.id.toString()}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    NotesDetailsBean notesDetailsBeanResponse = NotesDetailsBean.fromJson(map);
    if (notesDetailsBeanResponse.status) {
      notesDetailsBean = notesDetailsBeanResponse;
      notesListDetailsBean = notesDetailsBeanResponse.data;
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> onTaskAPICall() async {
    isLoader = false;

    setState(() {
      taskAPI();
      this.mounted;
    });
  }

  Future<void> onNotesAPICall() async {
    isLoader = false;

    setState(() {
      notesAPI();
      this.mounted;
    });
  }

  Future<void> accountScreenViewDetailsAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_ACCOUNT_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    AccountDetailsBean accountDetailsResponseBean =
        AccountDetailsBean.fromJson(map);

    if (accountDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });

      MyApp.callLoginAPI(context);
    }

    if (accountDetailsResponseBean.status) {
      accountDetailsBean = accountDetailsResponseBean;
      accountListBeanData = accountDetailsResponseBean.data;

      account = [
        for (int i = 0; i < accountListBeanData.length; i++)
          {
            'label': accountListBeanData[i].companyname.toString(),
            'value': accountListBeanData[i].id.toString(),
          },
      ];

      final jsonList = account.map((item) => jsonEncode(item)).toList();

      final uniqueJsonList = jsonList.toSet().toList();

      account = uniqueJsonList.map((item) => jsonDecode(item)).toList();

      account.insert(0, {
        'label': '',
        'value': '',
      });

      form = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'First Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': ownerId ?? "",
            'visibility': false,
          },
          {
            'apiKeyValue': 'firstname',
            'key': 'First Name',
            'type': 'input',
            'label': 'First Name',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'secondName',
            'key': 'Second Name',
            'type': 'input',
            'label': 'Second Name',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'statusid',
            'key': 'Status',
            'type': 'select',
            'label': 'Status',
            'required': true,
            'items': status != null
                ? status
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'accountid',
            'key': 'Account',
            'type': 'select',
            'label': 'Account ID',
            'required': true,
            'readOnly': false,
            'value': '',
            'items': account ??
                [
                  {'label': 'New', 'value': '1'},
                  {'label': 'Active', 'value': '2'},
                  {'label': 'Inactive', 'value': '3'},
                ],
          },
          {
            'apiKeyValue': 'title',
            'key': 'Title',
            'type': 'input',
            'label': 'Title',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'dept',
            'key': 'Department',
            'type': 'input',
            'label': 'Department',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'email',
            'key': 'e-mail',
            'type': 'input',
            'label': 'Email',
            'required': true,
            'readOnly': false,
            'validator': 'email',
            'value': "",
          },
          {
            'apiKeyValue': 'mobile',
            'key': 'mobile',
            'type': 'input',
            'label': 'Mobile no',
            'required': true,
            'readOnly': false,
            'validator': 'number',
            'value': "",
          },
          {
            'apiKeyValue': 'phone',
            'key': 'contact',
            'type': 'input',
            'label': 'Phone no',
            'required': true,
            'readOnly': false,
            'validator': 'number',
            'value': "",
          },
          {
            'apiKeyValue': 'area',
            'key': 'Area',
            'type': 'input',
            'label': 'Area',
            'required': true,
            'value': "",
          },
          {
            'apiKeyValue': 'address1',
            'key': 'Address Line 1',
            'type': 'input',
            'label': 'Address Line 1',
            'required': true,
            'value': "",
          },
          {
            'apiKeyValue': 'address2',
            'key': 'Address Line 2',
            'type': 'input',
            'label': 'Address Line 2',
            'required': false,
            'value': "",
          },
          {
            'apiKeyValue': 'state',
            'key': 'State',
            'type': 'input',
            'label': 'State ',
            'required': true,
            'value': "",
          },
        ]
      });

      setState(() {
        isLoader = false;
      });
    }
  }
}
