import 'dart:convert';

import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/dash_board_detail_bean.dart';
import 'package:example/Screens/CRM/DrawerMenuHeaderDesign.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:example/Screens/CRM/platform.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:transition/transition.dart';

class DashboardPage extends DrawerContent {
  DashboardPage({Key key, this.title});

  final String title;

  @override
  _DashboardPageState createState() => _DashboardPageState();
}

class _DashboardPageState extends State<DashboardPage> {
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;

  List<PopupMenuItem> popUpMenu = new List.empty(growable: true);
  SharedPreferences prefs;

  bool isLoader = true;
  num countValue = 2;

  num aspectWidth = 2;

  num aspectHeight = 1;

  List<Dashboard> dashboard = new List.empty(growable: true);

  bool isGridView = false;

  DashBoardDetailBean dashBoardDetailsBeanResponse;

  @override
  void initState() {
    colorCodePref();
    callDashBoardAPI();

    super.initState();
  }

  var now = DateTime.now();

  @override
  Widget build(BuildContext context) {
    SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle(
      statusBarColor: theme ?? AppColors.APP_COLOR_PRIMARY,
      //top bar color
      statusBarIconBrightness: Brightness.light,
      //top bar icons
      systemNavigationBarColor: Colors.black,
      //bottom bar color
      systemNavigationBarIconBrightness: Brightness.dark, //bottom bar icons
    ));
    return SafeArea(
      child: Scaffold(
        body: Center(
          child: Column(
            children: [
              DrawerMenuDesign(
                onMenuClicked: widget.onMenuPressed,
                isFilterEnabled: false,
              ),
              isLoader
                  ? Center(
                      child: Container(
                        margin: const EdgeInsets.only(top: 100.0),
                        alignment: Alignment.center,
                        child: Loader(
                          loaderColor: theme,
                        ),
                      ),
                    )
                  : RefreshIndicator(
                      onRefresh: () async {
                        setState(() async {
                          colorCodePref();
                          return await callDashBoardAPI();
                        });
                      },
                      color: theme,
                      child: Stack(
                        children: [
                          showGridView,
                          apiCallView,
                        ],
                      ),
                    ),
            ],
          ),
        ),
      ),
    );
  }

  Widget get showGridView {
    return Container(
      alignment: Alignment.topRight,
      padding: EdgeInsets.all(5),
      child: ClipRRect(
        borderRadius: BorderRadius.all(Radius.circular(32.0)),
        child: Material(
          shadowColor: Colors.transparent,
          color: Colors.transparent,
          child: IconButton(
            icon: Icon(
              !isGridView ? Icons.list : Icons.grid_view,
              semanticLabel:
                  isGridView ? "Change to List View" : "Change to Grid View",
              color: secondaryColor,
            ),
            onPressed: () {
              changeOrderView();
            },
          ),
        ),
      ),
    );
  }

  Widget get apiCallView {
    return Container(
      height: MediaQuery.of(context).size.height * 0.8,
      margin: EdgeInsets.only(top: 45),
      padding: EdgeInsets.only(top: 10, bottom: 20),
      child: GridView.count(
        crossAxisCount: countValue,
        childAspectRatio: (aspectWidth / aspectHeight),
        children: List.generate(
            dashBoardDetailsBeanResponse.data.dashboard.length, (index) {
          return BUICard.dashBoardCardDeals(
            cardTitle: dashboard[index].name.toString(),
            currentMonthPrice: dashboard[index].currentMonth.toString(),
            lastMonthPrice: dashboard[index].lastMonth.toString(),
            trendValue:
                dashboard[index].currentMonth > dashboard[index].lastMonth
                    ? true
                    : false,
            cardDate: "(" +
                "${MyApp.buiFormatter.getPreviousMonthNameAndYear(now.month)}" +
                ")",
            cardType: CardType.cardDashBoardDeal,
            cardSize: CardSize.small,
            width: MediaQuery.of(context).size.width,
            height: MediaQuery.of(context).size.height * 0.1,
            padding: 5,
            elevation: 10,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(5),
            ),
          );
        }),
      ),
    );
  }

  getGridViewSelectedItem(BuildContext context, String gridItem) {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          title: new Text(gridItem),
          actions: <Widget>[
            TextButton(
              child: new Text("OK"),
              onPressed: () {
                Navigator.of(context).pop();
              },
            ),
          ],
        );
      },
    );
  }

  Future<void> onMenuPressed() async {
    setState(() {
      widget.onMenuPressed();
    });
  }

  Future<void> changeOrderView() async {
    setState(() {
      isGridView = !isGridView;
    });

    changeMode();
  }

  changeMode() {
    if (countValue == 2) {
      setState(() {
        countValue = 1;
        aspectWidth = 3;
        aspectHeight = 1;
      });
    } else {
      setState(() {
        countValue = 2;
        aspectWidth = 2;
        aspectHeight = 1;
      });
    }
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return secondaryColorPref;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  void onBackPressed() {
    colorCodePref();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Message Pressed");
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {});
  }

  Future<void> onSettingFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Setting Pressed");
    });
  }

  Future<void> onNotificationFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Notification Pressed");
    });
  }

  Future<void> onPopUpMenuPressed(int count) async {
    switch (count) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
      context,
      Transition(
          child: ThemePickerScreen(
            onBackPressed: onBackPressed,
          ),
          transitionEffect: TransitionEffect.SCALE),
    );
  }

  Future<void> callDashBoardAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_DASHBOARD_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    DashBoardDetailBean dashBoardDetailBean = DashBoardDetailBean.fromJson(map);

    if (dashBoardDetailBean.statusCode == 401) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });

      print("Invalid Token====");
      MyApp.callLoginAPI(context);
    }
    if (dashBoardDetailBean.status) {
      dashBoardDetailsBeanResponse = dashBoardDetailBean;
      dashboard = dashBoardDetailBean.data.dashboard;
      setState(() {
        isLoader = false;
      });
      Utils.showSnackBar(context, dashBoardDetailBean.message);
    }
  }
}
