import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

class DrawerMenuDesign extends DrawerContent {
  Function() onMenuClicked;
  Function() isFilterClicked;

  bool isFilterEnabled;

  DrawerMenuDesign(
      {Key key,
      @required this.onMenuClicked,
      this.isFilterEnabled,
      this.isFilterClicked});

  @override
  DrawerMenuDesignState createState() => DrawerMenuDesignState();
}

class DrawerMenuDesignState extends State<DrawerMenuDesign> {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor = AppColors.APP_COLOR_PRIMARY;
  SharedPreferences prefs;

  @override
  void initState() {
    colorCodePref();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: primaryColor ?? AppColors.primaryColor,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          ClipRRect(
            borderRadius: BorderRadius.all(Radius.circular(32.0)),
            child: Material(
              shadowColor: Colors.transparent,
              color: Colors.transparent,
              child: IconButton(
                icon: Icon(
                  Icons.menu,
                  color: Colors.white,
                ),
                onPressed: () {
                  widget.onMenuClicked();
                },
              ),
            ),
          ),
          Row(
            children: [
              Visibility(
                visible: widget.isFilterEnabled ?? true,
                child: InkWell(
                  onTap: () {
                    widget.isFilterClicked();
                  },
                  child: Container(
                    //padding: EdgeInsets.only(right: 2),
                    child: Icon(
                      Icons.filter_alt,
                      color: AppColors.appWhite,
                    ),
                  ),
                ),
              ),
              Container(
                  //padding: EdgeInsets.only(right: 2),
                  child: PopupMenuButton(
                elevation: 30,
                //offset: Offset(0, 100),
                tooltip: "More Feature",
                itemBuilder: (context) => [
                  PopupMenuItem(
                    height: 30,
                    value: 1,
                    child: Row(
                      children: [
                        Padding(
                          padding: const EdgeInsets.all(12.0),
                          child: Icon(
                            Icons.settings,
                            color: primaryColor ?? AppColors.APP_COLOR_BLACK,
                          ),
                        ),
                        Text(
                          AppStrings.SETTINGS,
                          style: TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: FontSize.FONT_14,
                              color: primaryColor ?? Colors.black87),
                        )
                      ],
                    ),
                  ),
                  PopupMenuItem(
                    value: 2,
                    child: Row(
                      children: [
                        Padding(
                          padding: const EdgeInsets.all(12.0),
                          child: Icon(
                            Icons.notifications_active,
                            color: primaryColor ?? AppColors.APP_COLOR_BLACK,
                          ),
                        ),
                        Text(
                          AppStrings.NOTIFICATION,
                          style: TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: FontSize.FONT_14,
                              color: primaryColor ?? Colors.black87),
                        )
                      ],
                    ),
                  ),
                  PopupMenuItem(
                    value: 3,
                    child: Row(
                      children: [
                        Padding(
                          padding: const EdgeInsets.all(12.0),
                          child: Icon(
                            Icons.messenger_rounded,
                            color: primaryColor ?? AppColors.APP_COLOR_BLACK,
                          ),
                        ),
                        Text(
                          AppStrings.MESSAGE,
                          style: TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: FontSize.FONT_14,
                              color: primaryColor ?? Colors.black87),
                        )
                      ],
                    ),
                  ),
                  PopupMenuItem(
                    value: 4,
                    child: Row(
                      children: [
                        Padding(
                          padding: const EdgeInsets.all(12.0),
                          child: Icon(
                            Icons.color_lens_sharp,
                            color: primaryColor ?? AppColors.APP_COLOR_BLACK,
                          ),
                        ),
                        Text(
                          AppStrings.THEME,
                          style: TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: FontSize.FONT_14,
                              color: primaryColor ?? Colors.black87),
                        )
                      ],
                    ),
                  ),
                  PopupMenuItem(
                    value: 5,
                    child: Row(
                      children: [
                        Padding(
                          padding: const EdgeInsets.all(12.0),
                          child: Icon(
                            Icons.feedback_rounded,
                            color: primaryColor ?? AppColors.APP_COLOR_BLACK,
                          ),
                        ),
                        Text(
                          AppStrings.FEED_BACK,
                          style: TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: FontSize.FONT_14,
                              color: primaryColor ?? Colors.black87),
                        )
                      ],
                    ),
                  ),
                ],
                onCanceled: () {},
                onSelected: (value) {
                  onPopUpMenuFunction(value);
                },
                icon: Icon(
                  Icons.more_vert_sharp,
                  color: AppColors.appWhite,
                ),
              )),
            ],
          ),
        ],
      ),
    );
  }

  Future<void> onPopUpMenuFunction(int s) async {
    switch (s) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;
      case 6:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        Transition(
            child: ThemePickerScreen(
              onBackPressed: onBackPressed,
            ),
            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
  }

  void onBackPressed() {
    colorCodePref();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Toast.show("Message Pressed", context);
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {});
  }

  Future<void> onSettingFunction() async {
    setState(() {});
  }

  Future<void> onNotificationFunction() async {
    setState(() {});
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);
      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }
    String secondaryColorPf = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    if (secondaryColorPf != null) {
      int colorCodeInt = int.parse(secondaryColorPf);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });
    } else {
      setState(() {
        secondaryColor = AppColors.APP_COLOR_PRIMARY;
        this.mounted;
      });
    }
  }
}
