import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/ImageAssets.dart';
import 'package:example/Constants/fontFamily.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Firebase/sign_in.dart';
import 'package:example/Screens/CRM/AccountScreen.dart';
import 'package:example/Screens/CRM/ContactScreen.dart';
import 'package:example/Screens/CRM/DashboardPage.dart';
import 'package:example/Screens/CRM/LeadScreen.dart';
import 'package:example/Screens/CRM/TaskScreen.dart';
import 'package:example/Screens/ModuleSelectionScreen.dart';
import 'package:example/main.dart';
import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/CustomAlertComponent/CustomeAlert.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:universal_platform/universal_platform.dart';

class MainWidget extends StatefulWidget {
  MainWidget({Key key, this.title}) : super(key: key);
  final String title;

  @override
  MainWidgetState createState() => MainWidgetState();
}

class MainWidgetState extends State<MainWidget> with TickerProviderStateMixin {
  HiddenDrawerController _drawerController;
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
  SharedPreferences prefs;

  @override
  void initState() {
    super.initState();

    colorCodePrefs();
    _drawerController = HiddenDrawerController(
      initialPage: DashboardPage(
        title: 'main',
      ),
      items: [
        DrawerItem(
          text: Text(
            'Switch',
            style: TextStyle(
                color: Colors.white,
                fontSize: FontSize.FONT_15,
                fontFamily: FontFamily.Gilroy,
                fontStyle: FontStyle.normal,
                fontWeight: FontWeight.normal),
          ),
          icon: Icon(Icons.change_circle, color: Colors.white),
          onPressed: () {
            Navigator.of(context).push(
                MaterialPageRoute(builder: (_) => ModuleSelectionScreen()));
          },
          page: DashboardPage(
            title: 'main',
          ),
        ),
        DrawerItem(
          text: Text(
            'Dashboard',
            style: TextStyle(
                color: Colors.white,
                fontSize: FontSize.FONT_15,
                fontFamily: FontFamily.Gilroy,
                fontStyle: FontStyle.normal,
                fontWeight: FontWeight.normal),
          ),
          icon: Icon(Icons.dashboard, color: Colors.white),
          page: DashboardPage(
            title: 'Dashboard',
          ),
        ),
        DrawerItem(
          text: Text(
            'Leads',
            style: TextStyle(
                color: Colors.white,
                fontSize: FontSize.FONT_15,
                fontFamily: FontFamily.Gilroy,
                fontStyle: FontStyle.normal,
                fontWeight: FontWeight.normal),
          ),
          icon: Icon(Icons.leaderboard, color: Colors.white),
          page: LeadScreen(
            title: 'Leads',
          ),
        ),
        DrawerItem(
          text: Text(
            'Accounts',
            style: TextStyle(
                color: Colors.white,
                fontSize: FontSize.FONT_15,
                fontFamily: FontFamily.Gilroy,
                fontStyle: FontStyle.normal,
                fontWeight: FontWeight.normal),
          ),
          icon: Icon(Icons.account_circle_rounded, color: Colors.white),
          page: AccountScreen(
            title: 'Notification',
          ),
        ),
        DrawerItem(
          text: Text(
            'Contacts',
            style: TextStyle(
                color: Colors.white,
                fontSize: FontSize.FONT_15,
                fontFamily: FontFamily.Gilroy,
                fontStyle: FontStyle.normal,
                fontWeight: FontWeight.normal),
          ),
          icon: Icon(Icons.contact_page, color: Colors.white),
          page: ContactScreen(
            title: 'Contacts',
          ),
        ),
        DrawerItem(
          text: Text(
            'Task',
            style: TextStyle(
                color: Colors.white,
                fontSize: FontSize.FONT_15,
                fontFamily: FontFamily.Gilroy,
                fontStyle: FontStyle.normal,
                fontWeight: FontWeight.normal),
          ),
          icon: Icon(Icons.task, color: Colors.white),
          page: TaskScreen(
            title: 'Task',
          ),
        ),
      ],
    );
  }

  Future<void> colorCodePrefs() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);
      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }
    String secondaryColor = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    if (secondaryColor != null) {
      int colorCodeInt = int.parse(secondaryColor);
      setState(() {
        secondaryColorTheme = Color(colorCodeInt);
        this.mounted;
      });
    } else {
      setState(() {
        secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
        this.mounted;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: HiddenDrawer(
        controller: _drawerController,
        /*   header: Align(
          alignment: Alignment.bottomLeft,
          child: Column(
            children: <Widget>[
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: CircleAvatar(

                  radius: 50,
                  backgroundImage: NetworkImage(
                      'https://helpx.adobe.com/content/dam/help/en/photoshop/using/convert-color-image-black-white/jcr_content/main-pars/before_and_after/image-before/Landscape-Color.jpg'),
                ),
              ),
            ],
          ),
        ),*/
        header: Column(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Container(
              alignment: Alignment.topLeft,
              padding: EdgeInsets.only(left: 30),
              child:
                  /*CircleAvatar(
                radius: 50,
                backgroundImage: NetworkImage(
                    'https://helpx.adobe.com/content/dam/help/en/photoshop/using/convert-color-image-black-white/jcr_content/main-pars/before_and_after/image-before/Landscape-Color.jpg'),
              ),*/

                  CircleAvatar(
                radius: 70,
                backgroundColor: Colors.white,
                child: Container(
                  padding: EdgeInsets.all(20),
                  child: Image.asset(
                    ImageAssets.LOGO_WITH_NAME,
                    height: 150,
                  ),
                ),
              ),
            ),
          ],
        ),
        decoration: BoxDecoration(
          gradient: LinearGradient(
            begin: Alignment.topLeft,
            end: Alignment.bottomRight,
            colors: [
              /*ThemePickerScreenState.primaryColor,

              Colors.white,

              ThemePickerScreenState.primaryColor,*/
              Colors.black87,
              Colors.black87,
              Colors.black87,
            ],
            // tileMode: TileMode.repeated,
          ),
        ),
        logout: () {
          signOutFunction(context);
        },
        versionName: UniversalPlatform.isAndroid
            ? MyApp.version + AppStrings.COPY_RIGHT
            : UrlParams.AV_VALUE + AppStrings.COPY_RIGHT,
      ),
    );
  }

  static Future<void> signOutFunction(BuildContext context) async {
    CustomAlertBox.showCustomAlertBox(
        context: context,
        type: AppStrings.ARE_U_SURE,
        enterWidget: Text(AppStrings.DO_U_WANT_LOGOUT),
        isPositiveEnabled: true,
        isNegativeEnabled: true,
        positiveButtonColor: AppColors.primaryColor,
        setPositiveAction: () async {
          await FirebaseAuth.instance.signOut();
          await signOutGoogle(context);
          MyApp.user_token = "";
        },
        positiveButtonTextColor: Colors.white);
  }
}
