import 'dart:async';

import 'package:bui_component/SplashScreenComponent.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/ImageAssets.dart';
import 'package:flutter/material.dart';

class SplashScreen extends StatefulWidget {
  final int seconds;

  final Color backgroundColor;
  final dynamic navigateAfterSeconds;

  SplashScreen({
    @required this.seconds,
    this.navigateAfterSeconds,
    this.backgroundColor = Colors.white,
  });

  @override
  _SplashScreenState createState() => _SplashScreenState();
}

class MyCustomRoute<T> extends MaterialPageRoute<T> {
  MyCustomRoute({WidgetBuilder builder, RouteSettings settings})
      : super(builder: builder, settings: settings);

  @override
  Widget buildTransitions(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation, Widget child) {
    return child;
  }
}

class _SplashScreenState extends State<SplashScreen>
    with SingleTickerProviderStateMixin {
  AnimationController _animationController;
  Animation<double> _animation;

  @override
  void initState() {
    super.initState();
    _animationController = new AnimationController(
      vsync: this,
      duration: Duration(seconds: widget.seconds),
    );
    _animation = Tween(begin: 0.0, end: 1.0).animate(
        CurvedAnimation(parent: _animationController, curve: Curves.linear));
    Timer(Duration(seconds: widget.seconds), () {
      if (widget.navigateAfterSeconds is String) {
        return Navigator.pushReplacement(
          context,
          new MyCustomRoute(builder: (context) => widget.navigateAfterSeconds),
        );
      } else if (widget.navigateAfterSeconds is Widget) {
        return Navigator.of(context).pushReplacement(new MyCustomRoute(
            builder: (BuildContext context) => widget.navigateAfterSeconds));
      } else {
        throw new ArgumentError(
            'widget.navigateAfterSeconds must either be a String or Widget');
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: SplashScreenComponent(
        image: ImageAssets.LOGO_WITH_NAME,
        color: AppColors.APP_COLOR_PRIMARY,
        text: "Aagnia",
      ),
    );
  }
}
