import 'package:example/Constants/ImageAssets.dart';
import 'package:example/Screens/SignUpScreen.dart';
import 'package:example/Utils/Utility.dart';
import 'package:bui_component/LoginScreenComponent.dart';
import 'package:bui_component/Utils/SystemUtils.dart';
import 'package:flutter/material.dart';

import 'ForgotScreen.dart';

class LoginScreen extends StatefulWidget {
  @override
  _LoginScreenState createState() => _LoginScreenState();
}

class _LoginScreenState extends State<LoginScreen> {
  final passwordController = TextEditingController();
  final emailController = TextEditingController();
  var _formKey = GlobalKey<FormState>();

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: LoginScreenComponent(
        globalKey: _formKey,
        signInFunction: signInFunction,
        emailController: emailController,
        passwordController: passwordController,
        signUpFunction: signUpFunction,
        forgotFunction: forgotFunction,
        imageLogo: Container(
          height: 300,
          width: MediaQuery.of(context).size.width * 0.6,
          child: CircleAvatar(
            radius: 20,
            backgroundColor: Colors.white,
            child: Container(
              padding: EdgeInsets.all(20),
              child: Image.asset(
                ImageAssets.LOGO_WITH_NAME,
                height: 100,
              ),
            ),
          ),
        ),
      ),
    );
  }

  Future<void> signInFunction() async {
    final isValid = _formKey.currentState.validate();
    if (!isValid) {
      return;
    }
    _formKey.currentState.save();

    if (SystemUtils.hasMatchEmail(emailController.text.toString())) {
      Utils.showSnackBar(context, "Successfully Login");
    }
  }

  Future<void> forgotFunction() async {
    Navigator.push(context,
        MaterialPageRoute(builder: (BuildContext context) => ForgotScreen()));
  }

  Future<void> signUpFunction() async {
    Navigator.pushReplacement(context,
        MaterialPageRoute(builder: (BuildContext context) => SignUpScreen()));
  }
}
