import 'dart:convert';

import 'package:bui_component/JsonFormComponent/json_schema.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/PMO/leave_bean.dart';
import 'package:example/Models/PMO/time_sheet_bean.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/logger.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:toast/toast.dart';

class CommonViewDetailScreen extends StatefulWidget {
  final Color primaryColor;

  final Color secondaryColor;

  final String pageId;

  final String ownerId;

  List<dynamic> commonDD;
  List<dynamic> projectNameDD;

  LeaveListBean leaveListBean;
  TimeSheetListBean timeSheetListBean;
  List<TimeSheetListBean> addTimeSheetList;

  Function() onCallBack;

  CommonViewDetailScreen(
      {Key key,
        this.primaryColor,
        this.secondaryColor,
        this.pageId,
        this.ownerId,
        this.commonDD,
        this.projectNameDD,
        this.leaveListBean,
        this.timeSheetListBean,
        this.addTimeSheetList,
        this.onCallBack})
      : super(key: key);

  @override
  _CommonViewDetailScreenState createState() => _CommonViewDetailScreenState();
}

class _CommonViewDetailScreenState extends State<CommonViewDetailScreen> {
  String jsonForm;

  dynamic response;

  dynamic formGeneral;

  bool isLoader = false;

  Map keyBoardTypes = {
    'number': TextInputType.number,
    'text': TextInputType.text,
    'email': TextInputType.emailAddress,
    'password': TextInputType.visiblePassword,
  };

  @override
  void initState() {
    // TODO: implement initState
    super.initState();

    if (widget.pageId == AppStrings.timeSheetPage) {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'project_name',
            'key': 'select',
            'type': 'select',
            'label': 'Project Name',
            'required': true,
            'readOnly': false,
            'value': widget.timeSheetListBean.projectName.id.toString() ?? "",
            'items': widget.projectNameDD ??
                [
                  {'label': 'Development', 'value': '1'},
                  {'label': 'Design', 'value': '2'},
                  {'label': 'Management', 'value': '3'},
                  {'label': 'Specification', 'value': '4'},
                  {'label': 'Support', 'value': '5'},
                  {'label': 'Testing', 'value': '6'},
                  {'label': 'Other', 'value': '7'},
                ]
          },
          {
            'apiKeyValue': 'date',
            'key': 'dueDate',
            'type': 'date',
            'label': 'Date',
            'required': true,
            'value': MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(
                widget.timeSheetListBean.date.toString()) ??
                "",
          },
          {
            'apiKeyValue': 'hours',
            'key': 'number',
            'type': 'input',
            'label': 'Hours',
            'required': true,
            'readOnly': false,
            'value': widget.timeSheetListBean.hours.toString() ?? "",
          },
          {
            'apiKeyValue': 'proejct_description',
            'key': 'text',
            'type': 'input',
            'label': 'Project Comments',
            'required': true,
            'readOnly': false,
            'value':
            widget.timeSheetListBean.proejctDescription.toString() ?? "",
          },
          {
            'apiKeyValue': 'activity',
            'key': 'select',
            'type': 'select',
            'label': 'Activity',
            'required': true,
            'readOnly': false,
            'value': widget.timeSheetListBean.activity.id.toString() ?? "",
            'items': widget.commonDD ??
                [
                  {'label': 'Development', 'value': '1'},
                  {'label': 'Design', 'value': '2'},
                  {'label': 'Management', 'value': '3'},
                  {'label': 'Specification', 'value': '4'},
                  {'label': 'Support', 'value': '5'},
                  {'label': 'Testing', 'value': '6'},
                  {'label': 'Other', 'value': '7'},
                ]
          },
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
        ]
      });
    } else if (widget.pageId == AppStrings.leavePage) {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
          {
            'apiKeyValue': 'leave_type',
            'key': 'select',
            'type': 'select',
            'label': 'Leave Type',
            'required': true,
            'readOnly': false,
            'value': widget.leaveListBean.leaveType.id.toString() ?? "",
            'items': widget.commonDD ??
                [
                  {'label': 'Sick Leave', 'value': '1'},
                  {'label': 'Casual Leave', 'value': '2'},
                  {'label': 'LOP', 'value': '3'},
                ]
          },
          {
            'apiKeyValue': 'date',
            'key': 'dueDate',
            'type': 'date',
            'label': 'From Date',
            'required': true,
            'value': MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(
                widget.leaveListBean.date.toString()) ??
                DateTime.now().toString().substring(0, 10),
          },
          {
            'apiKeyValue': 'day',
            'key': 'number',
            'type': 'input',
            'label': 'Day(Ex: half day: 0.5)',
            'required': true,
            'readOnly': false,
            'value': widget.leaveListBean.day.toString() ?? "",
          },
          {
            'apiKeyValue': 'leave_description',
            'key': 'text',
            'type': 'input',
            'label': 'Leave Description',
            'required': true,
            'readOnly': false,
            'value': widget.leaveListBean.leaveDescription.toString() ?? "",
          },
        ]
      });
    } else if (widget.pageId == AppStrings.tech) {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
          {
            'apiKeyValue': 'owner_id',
            'key': 'text',
            'type': 'input',
            'label': 'Technologies',
            'required': true,
            'readOnly': false,
            'value': "",
            'visibility': true,
          },
        ]
      });
    } else if (widget.pageId == AppStrings.timesheetExp) {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
          {
            'apiKeyValue': 'date',
            'key': 'dueDate',
            'type': 'date',
            'label': 'Date',
            'required': true,
            'value':
            MyApp.buiFormatter.dateToStringDateReverse(DateTime.now()) ??
                "",
          },
        ]
      });
    } else if (widget.pageId == AppStrings.permission) {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
          {
            'apiKeyValue': 'name',
            'key': 'text',
            'type': 'input',
            'label': 'Name',
            'required': true,
            'readOnly': false,
            'value': "",
            'visibility': true,
          },
          {
            'apiKeyValue': 'roles',
            'key': 'users',
            'type': 'radioButton',
            'label': 'Add Permission to Roles',
            'items': [
              {
                'label': "Super Admin",
                'value': 1,
              },
              {
                'label': "Admin",
                'value': 2,
              },
              {
                'label': "User",
                'value': 3,
              },
              {
                'label': "Client",
                'value': 4,
              },
              {
                'label': "Vendor",
                'value': 5,
              }
            ]
          },
        ]
      });
    } else if (widget.pageId == AppStrings.users) {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
          {
            'apiKeyValue': 'name',
            'key': 'text',
            'type': 'input',
            'label': 'Name',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'email',
            'key': 'email',
            'type': 'input',
            'label': 'Email',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'roles',
            'key': 'users',
            'type': 'radioButton',
            'label': 'Users',
            'items': [
              {
                'label': "Super Admin",
                'value': 1,
              },
              {
                'label': "Admin",
                'value': 2,
              },
              {
                'label': "User",
                'value': 3,
              },
              {
                'label': "Client",
                'value': 4,
              },
              {
                'label': "Vendor",
                'value': 5,
              }
            ]
          },
          {
            'apiKeyValue': 'Password',
            'key': 'password',
            'type': 'input',
            'label': 'Password',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'cnfPassword',
            'key': 'password',
            'type': 'input',
            'label': 'Confirm Password',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'project_name',
            'key': 'select',
            'type': 'select',
            'label': 'Project Name',
            'required': true,
            'readOnly': false,
            'items': [
              {'label': 'Active', 'value': '2'},
              {'label': 'InActive', 'value': '3'},
            ]
          },
        ]
      });
    } else if (widget.pageId == AppStrings.roles) {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
          {
            'apiKeyValue': 'name',
            'key': 'text',
            'type': 'input',
            'label': 'Name',
            'required': true,
            'readOnly': false,
            'value': "",
            'visibility': true,
          },
          {
            'apiKeyValue': 'permission',
            'key': 'checkbox',
            'type': 'checkbox',
            'label': 'Assign Permission',
            'items': [
              {
                'label': "Read",
                'value': true,
              },
              {
                'label': "Write",
                'value': false,
              },
              {
                'label': "Edit",
                'value': false,
              },
              {
                'label': "Delete",
                'value': false,
              }
            ]
          },
        ]
      });
    } else if (widget.pageId == AppStrings.createLeavePage) {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
          {
            'apiKeyValue': 'leave_type',
            'key': 'select',
            'type': 'select',
            'label': 'Leave Type',
            'required': true,
            'readOnly': false,
            'value': "",
            'items': widget.commonDD ??
                [
                  {'label': 'Sick Leave', 'value': '1'},
                  {'label': 'Casual Leave', 'value': '2'},
                  {'label': 'LOP', 'value': '3'},
                ]
          },
          {
            'apiKeyValue': 'date',
            'key': 'dueDate',
            'type': 'date',
            'label': 'From Date',
            'required': true,
            'value': DateTime.now().toString().substring(0, 10).toString(),
          },
          {
            'apiKeyValue': 'day',
            'key': 'number',
            'type': 'input',
            'label': 'Day(Ex: half day: 0.5)',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'leave_description',
            'key': 'text',
            'type': 'input',
            'label': 'Leave Description',
            'required': true,
            'readOnly': false,
            'value': "",
          },
        ]
      });
    } else if (widget.pageId == AppStrings.createTimeSheetPage) {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'project_name',
            'key': 'select',
            'type': 'select',
            'label': 'Project Name',
            'required': true,
            'readOnly': false,
            'value': "",
            'items': widget.projectNameDD ??
                [
                  {'label': 'Development', 'value': '1'},
                  {'label': 'Design', 'value': '2'},
                  {'label': 'Management', 'value': '3'},
                  {'label': 'Specification', 'value': '4'},
                  {'label': 'Support', 'value': '5'},
                  {'label': 'Testing', 'value': '6'},
                  {'label': 'Other', 'value': '7'},
                ]
          },
          {
            'apiKeyValue': 'date',
            'key': 'dueDate',
            'type': 'date',
            'label': 'Date',
            'required': true,
            'value': DateTime.now().toString().substring(0, 10).toString(),
          },
          {
            'apiKeyValue': 'hours',
            'key': 'number',
            'type': 'input',
            'label': 'Hours',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'proejct_description',
            'key': 'text',
            'type': 'input',
            'label': 'Project Comments',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'activity',
            'key': 'select',
            'type': 'select',
            'label': 'Activity',
            'required': true,
            'readOnly': false,
            'value': "",
            'items': widget.commonDD ??
                [
                  {'label': 'Development', 'value': '1'},
                  {'label': 'Design', 'value': '2'},
                  {'label': 'Management', 'value': '3'},
                  {'label': 'Specification', 'value': '4'},
                  {'label': 'Support', 'value': '5'},
                  {'label': 'Testing', 'value': '6'},
                  {'label': 'Other', 'value': '7'},
                ]
          },
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
        ]
      });
    } else {
      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': false,
          },
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': widget.ownerId ?? "7",
            'visibility': true,
          },
        ]
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          backgroundColor: widget.primaryColor,
          //title: Text(AppStrings.create + widget.pageId),
        ),
        body: isLoader ? _isLoaderView : _mainView);
  }

  Widget get _mainView {
    return Container(
      child: ListView(
        children: [
          JsonSchema(
            keyboardTypes: keyBoardTypes,
            primaryColor: widget.primaryColor ?? AppColors.primaryColor,
            secondaryColor:
            widget.secondaryColor ?? AppColors.APP_COLOR_PRIMARY,
            form: jsonForm,
            onChanged: (dynamic response) {
              this.response = response;
            },
            buttonText: AppStrings.SAVE,
            actionSave: (data) {
              Utils.hideKeyboard(context);
              setState(() {
                formGeneral = data;
              });

              var jsonObj;

              if (widget.pageId == AppStrings.leavePage ||
                  widget.pageId == AppStrings.timeSheetPage) {
                jsonObj = {
                  for (int i = 0; i < data['fields'].length; i++)
                    json.encode(data['fields'][i]['apiKeyValue'].toString()):
                    json.encode(data['fields'][i]['value'].toString()),
                };
                updateAPIDetails(jsonObj.toString());
              } else if (widget.pageId == AppStrings.createLeavePage) {
                jsonObj = {
                  for (int i = 0; i < data['fields'].length; i++)
                    json.encode(data['fields'][i]['apiKeyValue'].toString()):
                    json.encode(data['fields'][i]['value'].toString()),
                  "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
                  "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
                  "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
                };
                applyLeaveAPI(jsonObj.toString());
              } else if (widget.pageId == AppStrings.createTimeSheetPage) {
                jsonObj = {
                  for (int i = 0; i < data['fields'].length; i++)
                    json.encode(data['fields'][i]['apiKeyValue'].toString()):
                    json.encode(data['fields'][i]['value'].toString()),
                  "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
                  "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
                  "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
                };
                applyTimeSheetAPI(jsonObj.toString());
              } else {
                print("Text" + jsonObj.toString());
              }

              Logger.logMessage(Log.DEBUG, jsonObj);
            },
            actionCancel: () {
              Navigator.pop(context);
            },
            buttonSave: true,
          ),
        ],
      ),
    );
  }

  Widget get _isLoaderView {
    return Center(
      child: Loader(
        loaderColor: widget.primaryColor,
      ),
    );
  }

  Future<void> updateAPIDetails(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController;

    var objToSend;
    if (widget.pageId == AppStrings.timeSheetPage) {
      urlController = ServerUrl.pmoTimeSheetUpdate;
      objToSend = {
        "\"condition\"": [
          {"\"id\"": widget.timeSheetListBean.id.toString()}
        ],
        "\"data\"": [obj.toString()],
        "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
        "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
        "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
      };
    } else {
      urlController = ServerUrl.pmoLeaveUpdate;
      objToSend = {
        "\"condition\"": [
          {"\"id\"": widget.leaveListBean.id.toString()}
        ],
        "\"data\"": [obj.toString()],
        "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
        "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
        "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
      };
    }

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: objToSend.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      widget.onCallBack();
      Navigator.pop(context);
      setState(() {
        isLoader = false;
      });
    } else {
      Toast.show(baseBean.message.toString(), context);
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> applyLeaveAPI(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoLeaveCreate;
    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: obj.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      widget.onCallBack();
      Navigator.pop(context);
      setState(() {
        isLoader = false;
      });
    } else {
      Toast.show(baseBean.message.toString(), context);
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> applyTimeSheetAPI(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoTimeSheetCreate;
    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: obj.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      widget.onCallBack();
      Navigator.pop(context);
      setState(() {
        isLoader = false;
      });
    } else {
      Toast.show(baseBean.message.toString(), context);
      setState(() {
        isLoader = false;
      });
    }
  }
}
