import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/CustomAlertComponent/CustomeAlert.dart';
import 'package:bui_component/JsonFormComponent/json_schema.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/PMO/leave_bean.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:example/Screens/NoDataFoundScreen.dart';
import 'package:example/Screens/PMO/CommonViewDetailScreen.dart';
import 'package:example/Screens/PMO/DrawerContent.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/loader.dart';
import 'package:example/Utils/logger.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

class LeavePage extends DrawerContent {
  String title;

  LeavePage({Key key, this.title});

  @override
  _LeavePageState createState() => _LeavePageState();
}

class _LeavePageState extends State<LeavePage> with TickerProviderStateMixin {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;
  SharedPreferences prefs;

  final List<Tab> myTabs = <Tab>[
    Tab(text: 'Leave Entry'),
    Tab(text: 'Leave List'),
  ];

  String jsonForm;
  dynamic formGeneral;
  dynamic response;

  List<dynamic> leaveDropDown = new List.empty(growable: true);

  String ownerId;

  LeaveBean leaveBeanResponse;

  List<LeaveListBean> leaveListBean = new List.empty(growable: true);

  bool isLoader = true;

  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();

  Map keyBoardTypes = {
    'input': TextInputType.number,
    'text': TextInputType.text,
  };

  @override
  void initState() {
    super.initState();

    masterView();

    leavePageList();
    //_mainTab = TabController(vsync: this, length: myTabs.length);
    colorCodePref();
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });

    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return secondaryColorPref;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        appBar: AppBar(
          backgroundColor: primaryColor,
          leading: InkWell(
              onTap: () {
                widget.onMenuPressed();
              },
              child: Icon(Icons.menu)),
          actions: <Widget>[
            Container(
                //padding: EdgeInsets.only(right: 2),
                child: PopupMenuButton(
              elevation: 30,
              //offset: Offset(0, 100),
              tooltip: "More Feature",
              itemBuilder: (context) => MainWidgetPMOState.popUpMenuItem,
              onCanceled: () {},
              onSelected: (value) {
                onPopUpMenuFunction(value);
              },
              icon: Icon(
                Icons.more_vert_sharp,
                color: AppColors.appWhite,
              ),
            )),
          ],
        ),
        key: _scaffoldKey,
        body: isLoader
            ? _isLoaderView
            : Column(
                children: [
                  deleteDetailsView,
                  _leaveList,
                ],
              ),
        floatingActionButton: FloatingActionButton(
          mini: true,
          onPressed: () {
            Navigator.push(
                context,
                Transition(
                    child: CommonViewDetailScreen(
                      primaryColor: primaryColor,
                      secondaryColor: secondaryColor,
                      pageId: AppStrings.createLeavePage,
                      ownerId: ownerId ?? "7",
                      commonDD: leaveDropDown,
                      onCallBack: () {
                        leavePageList();
                      },
                    ),
                    transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
          },
          child: const Icon(
            Icons.add,
          ),
          backgroundColor: primaryColor ?? Colors.green,
        ),
      ),
    );
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
        AppStrings.delete_and_edit,
        style: TextStyle(
            fontSize: 14, color: Colors.black87, fontWeight: FontWeight.bold),
      ),
      color: Colors.grey[400],
    );
  }

  Widget get _isLoaderView {
    return Center(
      child: Loader(
        loaderColor: primaryColor,
      ),
    );
  }

  Widget get _leaveList {
    if (leaveListBean.isNotEmpty) {
      return RefreshIndicator(
        onRefresh: () async {
          return await leavePageList();
        },
        color: primaryColor,
        child: Container(
          height: MediaQuery.of(context).size.height * 0.8,
          padding: EdgeInsets.only(top: 5, bottom: 30, left: 5, right: 5),
          color: AppColors.APP_COLOR_WHITE,
          child: ListView.builder(
            itemCount: leaveListBean.length,
            // Add one more item for progress indicator
            padding: EdgeInsets.symmetric(vertical: 8.0),
            itemBuilder: (BuildContext context, int index) {
              return Dismissible(
                key: Key(leaveListBean[index].leaveDescription.toString()),
                background: slideRightBackground(),
                secondaryBackground: slideLeftBackground(),
                confirmDismiss: (direction) async {
                  if (direction == DismissDirection.endToStart) {
                    final bool res = await CustomAlertBox.showCustomAlertBox(
                      context: context,
                      type: "Warning",
                      enterWidget: Text(
                          "Are you sure do you want to delete this ${leaveListBean[index].leaveType.name.toString()}"),
                      isPositiveEnabled: true,
                      isNegativeEnabled: true,
                      setPositiveAction: () {
                        /* setState(() {
                          leaveListBean.removeAt(index);
                        });
*/
                        leavePageDeleteAPI(leaveListBean[index].id.toString());
                      },
                    );
                    return res;
                  } else {
                    return Navigator.of(context).push(MaterialPageRoute(
                      builder: (context) {
                        return CommonViewDetailScreen(
                          primaryColor: primaryColor,
                          secondaryColor: secondaryColor,
                          pageId: AppStrings.leavePage,
                          ownerId: ownerId ?? "7",
                          commonDD: leaveDropDown,
                          leaveListBean: leaveListBean[index],
                          onCallBack: () {
                            leavePageList();
                            print("Call back calling====");
                          },
                        );
                      },
                    ));
                  }
                },
                child: InkWell(
                  onTap: () {
                    Navigator.of(context).push(Transition(
                        child: CommonViewDetailScreen(
                          primaryColor: primaryColor,
                          secondaryColor: secondaryColor,
                          pageId: AppStrings.leavePage,
                          ownerId: ownerId ?? "7",
                          commonDD: leaveDropDown,
                          leaveListBean: leaveListBean[index],
                          onCallBack: leavePageList,
                        ),
                        transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                  },
                  child: BUICard.dashBoardCard(
                    width: MediaQuery.of(context).size.width,
                    cardSize: CardSize.large,
                    cardTitle:
                        leaveListBean[index].leaveType.name.toString() ?? "",
                    cardName:
                        leaveListBean[index].leaveDescription.toString() ?? "",
                    statusTag: leaveListBean[index].day != 1
                        ? leaveListBean[index].day.toString() + AppStrings.days
                        : leaveListBean[index].day.toString() +
                                AppStrings.day ??
                            "",
                    dashBoardCardTime: MyApp.buiFormatter
                        .buiDateTimeDDMMYYFormat(
                            leaveListBean[index].date.toString()),
                    isTagValue: true,
                    tagValue: [],
                    height: 130,
                    elevation: 10,
                    shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(5.0),
                    ),
                  ),
                ),
              );
            },
          ),

          /*  child: BUICard.dashBoardCard(
          width: MediaQuery.of(context).size.width,
          cardSize: CardSize.large,
          cardTitle: "",
          cardName: "",
          statusTag: "",
          dashBoardCardTime: "Test",
          isTagValue: false,
          height: 130,
          elevation: 10,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(5.0),
          ),
        ),*/
        ),
      );
    } else {
      return NoDataFoundScreen();
    }
  }

  Widget slideLeftBackground() {
    return Container(
      color: Colors.red,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: <Widget>[
            Icon(
              Icons.delete,
              color: Colors.white,
            ),
            Text(
              " Delete",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.right,
            ),
            SizedBox(
              width: 20,
            ),
          ],
        ),
        alignment: Alignment.centerRight,
      ),
    );
  }

  Widget slideRightBackground() {
    return Container(
      color: Colors.green,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            SizedBox(
              width: 20,
            ),
            Icon(
              Icons.edit,
              color: Colors.white,
            ),
            Text(
              " Edit",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.left,
            ),
          ],
        ),
        alignment: Alignment.centerLeft,
      ),
    );
  }

  Future<void> leavePageDeleteAPI(String index) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoLeaveDelete;
    var objToSend = {
      "\"condition\"": [
        {
          "\"id\"": index,
        },
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse baseResponse = BaseResponse.fromJson(map);

    if (baseResponse.status) {
      leavePageList();
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
    Utils.hideKeyboard(context);
  }

  Future<void> leavePageList() async {
    print("Call back calling====2");
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoLeaveViewDetails;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    LeaveBean leaveBean = LeaveBean.fromJson(map);

    if (leaveBean.status) {
      leaveBeanResponse = leaveBean;
      leaveListBean = leaveBean.data;
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }

    setState(() {
      isLoader = false;
    });
  }

  Future<void> masterView() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoMaster;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);
    if (detailsBean.status) {
      leaveDropDown = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Leave")
            if (detailsBean.data[i].key == "Leave_Type")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];

      leaveDropDown.insert(0, {
        'label': '',
        'value': '',
      });

      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': ownerId ?? "7",
            'visibility': false,
          },
          {
            'apiKeyValue': 'leave_type',
            'key': 'select',
            'type': 'select',
            'label': 'Leave Type',
            'required': true,
            'readOnly': false,
            'value': "",
            'items': leaveDropDown ??
                [
                  {'label': 'Sick Leave', 'value': '1'},
                  {'label': 'Casual Leave', 'value': '2'},
                  {'label': 'LOP', 'value': '3'},
                ]
          },
          {
            'apiKeyValue': 'date',
            'key': 'dueDate',
            'type': 'date',
            'label': 'From Date',
            'required': true,
            'value': DateTime.now().toString().substring(0, 10).toString(),
          },
          {
            'apiKeyValue': 'day',
            'key': 'input',
            'type': 'input',
            'label': 'Day(Ex: half day: 0.5)',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'leave_description',
            'key': 'text',
            'type': 'input',
            'label': 'Leave Description',
            'required': true,
            'readOnly': false,
            'value': "",
          },
        ]
      });
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> applyLeaveAPI(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoLeaveCreate;
    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: obj.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      leavePageList();
      setState(() {
        isLoader = false;
      });
    } else {
      Toast.show(baseBean.message.toString(), context);
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> onPopUpMenuFunction(int value) async {
    switch (value) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;
      case 6:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        Transition(
            child: ThemePickerScreen(
              onBackPressed: onBackPressed,
            ),
            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
  }

  void onBackPressed() {
    colorCodePref();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Toast.show("Message Pressed", context);
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {});
  }

  Future<void> onSettingFunction() async {
    setState(() {});
  }

  Future<void> onNotificationFunction() async {
    setState(() {});
  }
}
