import 'dart:convert';
import 'dart:core';

import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/PMO/project_details_bean.dart';
import 'package:example/Models/PMO/time_sheet_bean.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Screens/CRM/DrawerMenuHeaderDesign.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:example/Screens/PMO/DrawerContent.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/logger.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/CustomAlertComponent/CustomeAlert.dart';
import 'package:bui_component/JsonFormComponent/json_schema.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

import 'CommonViewDetailScreen.dart';


class TimeSheetPage extends DrawerContent {
  String title;

  TimeSheetPage({Key key, this.title});

  @override
  _TimeSheetPageState createState() => _TimeSheetPageState();
}

class _TimeSheetPageState extends State<TimeSheetPage>
    with TickerProviderStateMixin {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;
  SharedPreferences prefs;

  final List<Tab> myTabs = <Tab>[
    Tab(text: 'Entry Timesheet'),
    Tab(text: 'Timesheet List'),
  ];

  List<dynamic> projectStatusDDList = new List.empty(growable: true);
  List<dynamic> projectNameDdList = new List.empty(growable: true);

  bool isLoader = true;

  String ownerId;

  String jsonForm;

  dynamic formGeneral;
  dynamic response;

  TimeSheetBean timeSheetReponse;

  List<TimeSheetListBean> timeSheetListBean = new List.empty(growable: true);

  int tabIndex = 0;

  Map keyBoardTypes = {
    'input': TextInputType.number,
    'text': TextInputType.text,
  };

  @override
  void initState() {
    super.initState();

    projectView();

    //_mainTab = TabController(vsync: this, length: myTabs.length);
    colorCodePref();
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
    prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return secondaryColorPref;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        appBar: AppBar(
          backgroundColor: primaryColor,
          leading: InkWell(
              onTap: () {
                widget.onMenuPressed();
              },
              child: Icon(Icons.menu)),
          actions: <Widget>[
            Container(
              //padding: EdgeInsets.only(right: 2),
                child: PopupMenuButton(
                  elevation: 30,
                  //offset: Offset(0, 100),
                  tooltip: "More Feature",
                  itemBuilder: (context) => MainWidgetPMOState.popUpMenuItem,
                  onCanceled: () {},
                  onSelected: (value) {
                    onPopUpMenuFunction(value);
                  },
                  icon: Icon(
                    Icons.more_vert_sharp,
                    color: AppColors.appWhite,
                  ),
                )),
          ],
        ),
        body: isLoader
            ? _isLoaderView
            : Column(children: [
          deleteDetailsView,
          _timeSheetList,
        ]),
        floatingActionButton: FloatingActionButton(
          mini: true,
          onPressed: () {
            Navigator.push(
                context,
                Transition(
                    child: CommonViewDetailScreen(
                      primaryColor: primaryColor,
                      secondaryColor: secondaryColor,
                      pageId: AppStrings.createTimeSheetPage,
                      ownerId: ownerId ?? "7",
                      commonDD: projectStatusDDList,
                      projectNameDD: projectNameDdList,
                      onCallBack: timeSheetList,
                    ),
                    transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
          },
          child: const Icon(
            Icons.add,
          ),
          backgroundColor: primaryColor ?? Colors.green,
        ),
      ),
    );
  }

  Widget get _isLoaderView {
    return Center(
      child: Loader(
        loaderColor: primaryColor,
      ),
    );
  }

  Widget get _timeSheetList {
    if (timeSheetListBean.isNotEmpty) {
      return RefreshIndicator(
        color: primaryColor,
        onRefresh: () async {
          return await projectView();
        },
        child: Container(
          height: MediaQuery.of(context).size.height * 0.8,
          padding: EdgeInsets.only(top: 5, bottom: 30, left: 5, right: 5),
          color: AppColors.APP_COLOR_WHITE,
          child: ListView.builder(
            itemCount: timeSheetListBean.length,
            // Add one more item for progress indicator
            padding: EdgeInsets.symmetric(vertical: 8.0),
            itemBuilder: (BuildContext context, int index) {
              return Dismissible(
                key: Key(index.toString()),
                background: slideRightBackground(),
                secondaryBackground: slideLeftBackground(),
                confirmDismiss: (direction) async {
                  if (direction == DismissDirection.endToStart) {
                    final bool res = await CustomAlertBox.showCustomAlertBox(
                      context: context,
                      type: "Warning",
                      enterWidget: Text(
                          "Are you sure do you want to delete this ${timeSheetListBean[index].projectName.projectName + index.toString()}"),
                      isPositiveEnabled: true,
                      isNegativeEnabled: true,
                      setPositiveAction: () {
                        /*setState(() {
                          timeSheetListBean.removeAt(index);
                        });*/
                        timeSheetDeleteAPI(
                            timeSheetListBean[index].id.toString());
                      },
                    );
                    return res;
                  } else {
                    return Navigator.of(context).push(MaterialPageRoute(
                      builder: (context) {
                        return CommonViewDetailScreen(
                          primaryColor: primaryColor,
                          secondaryColor: secondaryColor,
                          pageId: AppStrings.timeSheetPage,
                          ownerId: ownerId ?? "7",
                          commonDD: projectStatusDDList,
                          projectNameDD: projectNameDdList,
                          timeSheetListBean: timeSheetListBean[index],
                          onCallBack: timeSheetList,
                        );
                      },
                    ));
                  }
                },
                child: InkWell(
                  onTap: () {
                    Navigator.of(context).push(Transition(
                        child: CommonViewDetailScreen(
                          primaryColor: primaryColor,
                          secondaryColor: secondaryColor,
                          pageId: AppStrings.timeSheetPage,
                          ownerId: ownerId ?? "7",
                          commonDD: projectStatusDDList,
                          projectNameDD: projectNameDdList,
                          timeSheetListBean: timeSheetListBean[index],
                          onCallBack: timeSheetList,
                        ),
                        transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                  },
                  child: BUICard.dashBoardCard(
                    width: MediaQuery.of(context).size.width,
                    cardSize: CardSize.large,
                    cardTitle: timeSheetListBean[index]
                        .projectName
                        .projectName
                        .toString() ??
                        "",
                    cardName:
                    timeSheetListBean[index].activity.name.toString() ?? "",
                    statusTag: timeSheetListBean[index].hours.toString() +
                        AppStrings.hours ??
                        "",
                    dashBoardCardTime: MyApp.buiFormatter
                        .buiDateTimeDDMMYYFormat(
                        timeSheetListBean[index].date.toString()) ??
                        "",
                    isTagValue: true,
                    tagValue: [],
                    height: 130,
                    elevation: 10,
                    shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(5.0),
                    ),
                  ),
                ),
              );
            },
          ),
        ),
      );
    } else {
      return Container(
        color: Colors.redAccent,
      );
    }
  }

  Widget slideLeftBackground() {
    return Container(
      color: Colors.red,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: <Widget>[
            Icon(
              Icons.delete,
              color: Colors.white,
            ),
            Text(
              " Delete",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.right,
            ),
            SizedBox(
              width: 20,
            ),
          ],
        ),
        alignment: Alignment.centerRight,
      ),
    );
  }

  Widget slideRightBackground() {
    return Container(
      color: Colors.green,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            SizedBox(
              width: 20,
            ),
            Icon(
              Icons.edit,
              color: Colors.white,
            ),
            Text(
              " Edit",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.left,
            ),
          ],
        ),
        alignment: Alignment.centerLeft,
      ),
    );
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
        AppStrings.delete_and_edit,
        style: TextStyle(
            fontSize: 14, color: Colors.black87, fontWeight: FontWeight.bold),
      ),
      color: Colors.grey[400],
    );
  }

  Future<void> masterView() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoMaster;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);
    if (detailsBean.status) {
      projectStatusDDList = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Timesheet")
            if (detailsBean.data[i].key == "Activity")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];

      projectStatusDDList.insert(0, {
        'label': '',
        'value': '',
      });

      jsonForm = json.encode({
        'fields': [
          {
            'apiKeyValue': 'project_name',
            'key': 'select',
            'type': 'select',
            'label': 'Project Name',
            'required': true,
            'readOnly': false,
            'value': "",
            'items': projectNameDdList ??
                [
                  {'label': 'Development', 'value': '1'},
                  {'label': 'Design', 'value': '2'},
                  {'label': 'Management', 'value': '3'},
                  {'label': 'Specification', 'value': '4'},
                  {'label': 'Support', 'value': '5'},
                  {'label': 'Testing', 'value': '6'},
                  {'label': 'Other', 'value': '7'},
                ]
          },
          {
            'apiKeyValue': 'date',
            'key': 'dueDate',
            'type': 'date',
            'label': 'Date',
            'required': true,
            'value': DateTime.now().toString().substring(0, 10).toString(),
          },
          {
            'apiKeyValue': 'hours',
            'key': 'input',
            'type': 'input',
            'label': 'Hours',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'proejct_description',
            'key': 'text',
            'type': 'input',
            'label': 'Project Comments',
            'required': true,
            'readOnly': false,
            'value': "",
          },
          {
            'apiKeyValue': 'activity',
            'key': 'select',
            'type': 'select',
            'label': 'Activity',
            'required': true,
            'readOnly': false,
            'value': "",
            'items': projectStatusDDList ??
                [
                  {'label': 'Development', 'value': '1'},
                  {'label': 'Design', 'value': '2'},
                  {'label': 'Management', 'value': '3'},
                  {'label': 'Specification', 'value': '4'},
                  {'label': 'Support', 'value': '5'},
                  {'label': 'Testing', 'value': '6'},
                  {'label': 'Other', 'value': '7'},
                ]
          },
          {
            'apiKeyValue': 'owner_id',
            'key': 'Company Name',
            'type': 'input',
            'label': 'Owner ID',
            'required': true,
            'readOnly': true,
            'value': ownerId ?? "7",
            'visibility': false,
          },
        ]
      });
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> projectView() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoProjectView;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    ProjectDetailsBean projectDetailsBean = ProjectDetailsBean.fromJson(map);
    if (projectDetailsBean.status) {
      projectNameDdList = [
        for (int i = 0; i < projectDetailsBean.data.length; i++)
          {
            'label': projectDetailsBean.data[i].projectName,
            'value': projectDetailsBean.data[i].id.toString(),
          },
      ];

      projectNameDdList.insert(0, {
        'label': '',
        'value': '',
      });
    }

    masterView();
    timeSheetList();
  }

  Future<void> applyTimeSheetAPI(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoTimeSheetCreate;
    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: obj.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      timeSheetList();
      setState(() {
        isLoader = false;
      });
    } else {
      Toast.show(baseBean.message.toString(), context);
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> timeSheetList() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoTimeSheetViewDetails;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TimeSheetBean timeSheetBean = TimeSheetBean.fromJson(map);

    if (timeSheetBean.status) {
      timeSheetReponse = timeSheetBean;
      timeSheetListBean = timeSheetBean.data;
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> timeSheetDeleteAPI(String index) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoTimeSheetDelete;
    var objToSend = {
      "\"condition\"": [
        {
          "\"id\"": index,
        },
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse baseResponse = BaseResponse.fromJson(map);

    if (baseResponse.status) {
      timeSheetList();
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
    Utils.hideKeyboard(context);
  }

  Future<void> onPopUpMenuFunction(int value) async {
    switch (value) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;
      case 6:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        Transition(
            child: ThemePickerScreen(
              onBackPressed: onBackPressed,
            ),
            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
  }

  void onBackPressed() {
    colorCodePref();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Toast.show("Message Pressed", context);
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {});
  }

  Future<void> onSettingFunction() async {
    setState(() {});
  }

  Future<void> onNotificationFunction() async {
    setState(() {});
  }
}
