import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/ImageAssets.dart';
import 'package:example/Constants/SvgPictures.dart';
import 'package:example/Screens/CRM/MenuPage.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:example/Screens/CommonTestingModule/demo1.dart';
import 'package:example/Screens/PMO/DrawerContent.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

class ModuleTestingScreen extends StatefulWidget {
  const ModuleTestingScreen({Key key}) : super(key: key);

  @override
  ModuleTestingScreenState createState() => ModuleTestingScreenState();
}

class ModuleTestingScreenState extends State<ModuleTestingScreen> {
  List<ModuleList> moduleList = [
    ModuleList(name: "CRM", id: "1", isEnabled: true,svgPicture: SvgPictures.FB_SVG),
    ModuleList(name: "PMO", id: "2", isEnabled: true,svgPicture: SvgPictures.DOG),
    ModuleList(name: "ERP", id: "3", isEnabled: false,svgPicture: SvgPictures.GOOGLE_SVG),
  ];

  SharedPreferences prefs;

  Color primaryColor = AppColors.APP_COLOR_PRIMARY;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();

    colorCodePref();
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();

    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }
  }

  bool isEnabled = true;

  double radius = 10;
  double width = 140;
  double height = 140;
  double elevation = 5;

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        appBar: AppBar(
          elevation: 1.0,
          backgroundColor: Colors.white,
          title: Text(AppStrings.selectModule,
              style: TextStyle(
                  color: Colors.black,
                  fontWeight: FontWeight.w700,
                  fontSize: FontSize.FONT_14)),
          actions: <Widget>[
            Container(
              margin: EdgeInsets.only(right: 8.0),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: <Widget>[
                  Container(
                    padding: EdgeInsets.all(8),
                    height: 120,
                    child: Image.asset(ImageAssets.LOGO_WITH_NAME),
                  ),
                  IconButton(
                    icon: Icon(Icons.logout),
                    color: AppColors.primaryColor,
                    onPressed: () {
                      print("On Pressed===");
                    },
                  ),
                ],
              ),
            )
          ],
        ),
        //backgroundColor: Colors.black,
        body: GridView.builder(
            gridDelegate: SliverGridDelegateWithMaxCrossAxisExtent(
                maxCrossAxisExtent: 200,
                childAspectRatio: 3 / 2,
                crossAxisSpacing: 1,
                mainAxisSpacing: 1),
            itemCount: moduleList.length,
            itemBuilder: (BuildContext ctx, index) {
              return InkWell(
                radius: 30,
                enableFeedback: true,
                splashColor: AppColors.halfWhite,
                onTap: () {
                  if (moduleList[index].name == "CRM") {
                    if (moduleList[index].isEnabled) {
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (context) {
                            return MainWidget();
                          },
                        ),
                      );
                    } else {
                      Toast.show("Not Access to this module", context);
                    }
                  } else if (moduleList[index].name == "ERP") {
                    if (moduleList[index].isEnabled) {
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (context) {
                            return AllFields();
                          },
                        ),
                      );
                    } else {
                      Toast.show("Not Access to this module", context);
                    }
                  } else if (moduleList[index].name == "PMO") {
                    if (moduleList[index].isEnabled) {
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (context) {
                            return MainWidgetPMO();
                          },
                        ),
                      );
                    } else {
                      Toast.show("Not Access to this module", context);
                    }
                  } else {
                    if (moduleList[index].isEnabled) {
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (context) {
                            return MainWidgetPMO();
                          },
                        ),
                      );
                    } else {
                      Toast.show("Not Access to this module", context);
                    }
                  }
                },
                child: Container(
                    alignment: Alignment.center,
                    padding: EdgeInsets.all(8),
                    width: width,
                    height: height,
                    child:

                    BUICard.imageWithTextCard(
                      isVisible: true,
                      radius: 10,
                      isEnabled: moduleList[index].isEnabled,
                      cardColor: primaryColor,
                      gradientColor: [
                        primaryColor,
                        primaryColor,
                        Colors.grey.shade50.withOpacity(1)
                      ],
                      cardText: moduleList[index].name,
                      cardTextColor: AppColors.APP_COLOR_WHITE,
                      cardType: CardType.cardWithImageAndText,
                      svgImage: moduleList[index].svgPicture,
                    )



                  /*Card(
                      color: moduleList[index].isEnabled
                          ? AppColors.primaryColor
                          : Colors.grey,
                      elevation: elevation,
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(radius),
                      ),
                      child: Stack(
                        children: <Widget>[
                          Align(
                            alignment: Alignment.topCenter,
                            child: Container(
                              margin: EdgeInsets.only(top: 15),
                              child: SvgPicture.asset(
                                SvgPictures.DOG,
                                width: 35,
                                height: 35,
                              ),
                            ),
                          ),
                          Align(
                            alignment: Alignment.bottomCenter,
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.stretch,
                              verticalDirection: VerticalDirection.up,
                              children: <Widget>[
                                Container(
                                  padding: EdgeInsets.all(10),
                                  decoration: BoxDecoration(
                                    *//* gradient: LinearGradient(
                                  colors: [Colors.deepOrange, Colors.yellow],
                                  stops: [0.0, 0.7],
                                ),*//*
                                    color: moduleList[index].isEnabled
                                        ? Colors.white.withOpacity(0.4)
                                        : Colors.white,
                                    borderRadius: BorderRadius.only(
                                        bottomLeft: Radius.circular(radius),
                                        bottomRight: Radius.circular(radius)),
                                  ),
                                  alignment: Alignment.center,
                                  child: Text(
                                    moduleList[index].name,
                                    style: TextStyle(
                                        color: moduleList[index].isEnabled
                                            ? Colors.white
                                            : Colors.black.withOpacity(0.4),
                                        fontSize: 14,
                                        fontWeight: FontWeight.bold),
                                  ),
                                )
                              ],
                            ),
                          ),
                        ],
                      ),
                    )*/

                ),


              );
            }),
      ),
    );
  }

  Widget get _secondWidget {
    return Scaffold(
      appBar: AppBar(
        title: Text("Select Your Option"),
        backwardsCompatibility: false,
        automaticallyImplyLeading: false,
        backgroundColor: AppColors.halfWhite,
        actions: [
          Container(
            padding: EdgeInsets.all(10),
            child: IconButton(
                onPressed: () {
                  Navigator.push(
                      context,
                      Transition(
                          child: ThemePickerScreen(
                            onBackPressed: colorCodePref,
                          ),
                          transitionEffect: TransitionEffect.SCALE));
                },
                icon: Icon(
                  Icons.color_lens_outlined,
                  color: AppColors.appWhite,
                )),
          )
        ],
      ),
      body: Container(
        alignment: Alignment.center,
        width: MediaQuery.of(context).size.width,
        padding: EdgeInsets.all(5),
        child: GridView.builder(
            gridDelegate: SliverGridDelegateWithMaxCrossAxisExtent(
                maxCrossAxisExtent: 200,
                childAspectRatio: 3 / 2,
                crossAxisSpacing: 1,
                mainAxisSpacing: 1),
            itemCount: moduleList.length,
            itemBuilder: (BuildContext ctx, index) {
              return InkWell(
                  radius: 30,
                  enableFeedback: true,
                  splashColor: AppColors.halfWhite,
                  onTap: () {
                    if (moduleList[index].name == "CRM") {
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (context) {
                            return MainWidget();
                          },
                        ),
                      );
                    } else if (moduleList[index].name == "ERP") {
                      /*Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (context) {
                            return QRScanner();
                          },
                        ),
                      );*/
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (context) {
                            return AllFields();
                          },
                        ),
                      );
                    } else {
                      Navigator.of(context).push(
                        MaterialPageRoute(
                          builder: (context) {
                            return MainWidgetPMO();
                          },
                        ),
                      );
                    }
                  },
                  child: BUICard.imageWithTextCard(
                    isVisible: false,
                    gradientColor: [
                      primaryColor,
                      primaryColor,
                      Colors.grey.shade50.withOpacity(1)
                    ],
                    cardText: moduleList[index].name,
                    cardTextColor: AppColors.APP_COLOR_WHITE,
                    cardType: CardType.cardWithImageAndText,
                    svgImage: SvgPictures.FB_SVG,
                  ));
            }),
      ),
    );
  }
}

class ModuleList {
  String name;
  String id;
  bool isEnabled = true;

  String svgPicture;

  ModuleList({this.name, this.id, this.isEnabled,this.svgPicture});
}
